function [supLM, crit] = BE_test(Yobs, r, tau)
%% This function calculate Breitung and Eickmeier's (2011, JOE) test for
%% structural break in factor models 
%% Witten by Xia Wang,  Dec. 2014, @SMU, Singapore
%% Input: yobs: data, dimension: T x N 
%%             r: number of factor 
%%            tau: trimming parameter 

%% Output: supLM: supLM test statistic , dim: 1 x N 
%%                crit: The corresponding Andrews's (1993) critical value [5%, 10%], dim: 1 x 2  

%% Step 1: Estimate common factor and factor loadings by PCA 
%%       Yobs = F * lam' + e, Yobs: T x N;   F: T x r;    lam: N x r
[T, N] = size(Yobs); 
YY = Yobs * Yobs' /(T*N);                      % dim: T x T 
[Ffit0, D] = eig(YY); 
F0 = Ffit0(:, T-r+1:T) * sqrt(T);              % dim: T x r, estimated factors by PCA 
lam0 = Yobs' * F0/T;                             % dim: T x r, estimated factor loadings 
eps = Yobs - F0 * lam0';                      % error term, dim: T x N 

%% Step 2: Run the regression: 
%%                           eps = theta*F_t + phi* F_t* + e
%%                          get the goodness of Fit, R2 
%% get LM and supLM 

T0 = floor(tau*T)+1;      % started date 
T1 = floor((1-tau)*T)+1;  % ended date 
LM = zeros(T1-T0+1, N); 

for t = T0 : T1 
    Fstar = zeros(T, r); 
    Fstar(t+1:T, :) = F0(t+1:T,:); 
    F = [F0, Fstar];           % dim: T x 2r 
    beta = pinv(F' * F) * F' * eps;       % dim: 2r * N
    e = eps - F * beta;                        % dim: T x N 
    % calculate goodness of fit 
    R2 = 1- sum(e.^2, 1)./ sum(eps.^2, 1);   % dim: 1x N 
    LM ( t-T0+1, :) = T * R2; 
end 

supLM = max(LM, [], 1);                %dim: 1 x N 

%% Step 3: Get Andrews' (1993) critical value 
% we only import tau = 0.25. 0.20, 0.15, 0.10, 0.05 
%                           r = 1, 2 3, .....8 

CritValue5 = [ 7.93, 10.75, 13.16, 15.34, 17.25, 19.07, 20.63, 20.95;
                         8.45, 11.26, 13.69, 15.84, 17.88, 19.64, 21.07, 21.47;
                         8.85, 11.79, 14.15, 16.45, 18.35, 20.26, 21.84, 22.13;
                         9.31, 12.27, 14.62, 16.98, 18.93, 20.82, 22.51, 22.87;
                         9.84, 12.93, 15.15, 17.56, 19.61, 21.56, 23.22, 23.60 ];
                     
 CritValue10 = [6.46, 9.10, 11.29,13.36, 15.17, 17.00, 18.38, 18.61;
                           6.80, 9.59, 11.80, 13.82, 15.63, 17.56, 19.04, 19.17;
                           7.17,10.01, 12.27, 14.31, 16.20, 18.12, 19.69, 19.82;
                           7.63, 10.50, 12.81, 14.94, 16.87, 18.78, 20.32, 20.45;
                           8.19, 11.20, 13.47, 15.62, 17.69, 19.49, 21.02, 21.23 ];
                       
 kk = round((0.3 - tau)/0.05) ;
 
 crit = [CritValue5(kk, r), CritValue10(kk,r)]; 
                       















