function[supLM, supW, crit] = CDG_test(Yobs, r, tau)
%% This function calculate Chen, Dolado and Gonzalo's (2014, JOE) test for
%% big structural breaks in large factor models 
%% Witten by Xia Wang,  Dec. 2014, @SMU, Singapore
%% Input: yobs: data, dimension: T x N 
%%             r: number of factor, should >=2 
%%            tau: trimming parameter 

%% Output: supLM: supLM test statistic , dim: 1 x 1
%%                supW: supW test statistic, dim: 1 x 1
%%                crit: The corresponding Andrews's (1993) critical value [5%, 10%], dim: 1 x 2  

[T, N] = size(Yobs); 
%% Step 1: Estimate common factor and factor loadings by PCA 
%%       Yobs = F * lam' + e, Yobs: T x N;   F: T x r;    lam: N x r
[T, N] = size(Yobs); 
YY = Yobs * Yobs' /(T*N);                      % dim: T x T 
[Ffit0, D] = eig(YY); 
F = Ffit0(:, T-r+1:T) * sqrt(T);              % dim: T x r, estimated factors by PCA 
lam = Yobs' * F/T;                             % dim: T x r, estimated factor loadings 

%% Step 2: Regress F1 on the reminding factors Fo = [F2, ...Fr]
F1 = F(:,1);                                 % dim: T x 1 
Fo = F(:, 2:r);                              % dim: T x (r-1) 
b = pinv(Fo' * Fo) * Fo' * F1;      % dim: (r-1) x 1 
u = F1  - Fo * b;                         % dim: T x 1 

%% Step 3: Newey and West's (1987) HAC estimator of covariance matrix S
% m = round(T^(1/5));            % remark: NW require m = o(T^{1/4}), and CDG don't give the exact value of m in the similation and application part 
 m = round(T^(1/3));  
w = 1- [1: 1: m]' /(m);        % dim: m x 1, Bartlett kernel 

S = zeros(r-1, r-1); 
S = (Fo .* (F1 * ones(1, r-1)) )'  *  (Fo .* (F1 * ones(1, r-1)) )/T;     % dim: (r-1) x (r-1) 

for j = 1: m 
    Gj = (Fo(j+1: T, :)  .* (F1(j+1: T,:)  * ones(1, r-1)) )'  *  (Fo(1: T-j,:) .* (F1(1: T-j,:) * ones(1, r-1)) )/T ;
    S = S + w(j) * (Gj+ Gj') ; 
end 

%% Step 4: Construct supLM and sup-Wald test statistics 
T0 = floor(tau*T)+1;      % started date 
T1 = floor((1-tau)*T)+1;  % ended date 
LM = zeros(T1-T0+1, 1); 
Wald = zeros(T1-T0+1, 1); 


for t = T0 : T1 
     p = t/T; 
     Fu = sum( Fo(1: t,:) .* (u(1:t,:) * ones(1, r-1)) , 1)/sqrt(T);         % dim: 1 x (r-1) 
     LMt = Fu * pinv(S) * Fu' /(p * (1-p)); 
     LM(t-T0+1) = LMt; 
     c1 = pinv(Fo(1: t,:)' * Fo(1: t,:)) * Fo(1: t,:)' * F1(1: t,:);      % dim: (r-1) x 1 
     c2 = pinv(Fo(t+1: T,:)' * Fo(t+1:T,:)) * Fo(t+1:T ,:)' * F1(t+1:T, :);      % dim: (r-1) x 1
     Wt = T * p * (1-p) * (c1 - c2)' * pinv(S) * (c1-c2); 
     Wald(t-T0+1) = Wt;  
end 

supLM = max(LM);                %dim: 1 x 1
supW = max(Wald);              %dim: 1 x 1 

%% Step 5: Get Andrews' (1993) critical value 
% we only import tau = 0.25. 0.20, 0.15, 0.10, 0.05 
%                           r = 1, 2 3, .....8 

CritValue5 = [ 7.93, 10.75, 13.16, 15.34, 17.25, 19.07, 20.63, 20.95;
                         8.45, 11.26, 13.69, 15.84, 17.88, 19.64, 21.07, 21.47;
                         8.85, 11.79, 14.15, 16.45, 18.35, 20.26, 21.84, 22.13;
                         9.31, 12.27, 14.62, 16.98, 18.93, 20.82, 22.51, 22.87;
                         9.84, 12.93, 15.15, 17.56, 19.61, 21.56, 23.22, 23.60 ];
                     
 CritValue10 = [6.46, 9.10, 11.29,13.36, 15.17, 17.00, 18.38, 18.61;
                           6.80, 9.59, 11.80, 13.82, 15.63, 17.56, 19.04, 19.17;
                           7.17,10.01, 12.27, 14.31, 16.20, 18.12, 19.69, 19.82;
                           7.63, 10.50, 12.81, 14.94, 16.87, 18.78, 20.32, 20.45;
                           8.19, 11.20, 13.47, 15.62, 17.69, 19.49, 21.02, 21.23 ];
                       
 kk = round((0.3 - tau)/0.05) ;
 if r >=2 
     crit = [CritValue5(kk, r-1), CritValue10(kk,r-1)]; 
 else
     crit = ones(1,2)*100; 
 end 
















