function [ X, Ftrue, lam ] = DGP_determine_factors(N, T, h, DGP, lam, sig )
%% This function is used to generate datas for the simulation studies of the paper 
%% Testing for structural changes in factor models via a nonparametric regreesion
%% The basic model:  %% Xit = lam_it * Ft + e_it  
%% Written by Xia WANG, Dec. 17, 2014, @SMU, Singapore
%% Input: lam: Nx r or T x N x r; sig: N x 1 
%% Output: X: T x N; Ftrue: T x r, lam: N x r or T x N x r 

if DGP == 1
   %% lam_i ~ i.i.d.N(0,1) fixed for each rep, so input lam 
   %% Ft = alp* F_{t-1} + u_it, u_it~ i.i.d.N(0,1) 
   %% e_it ~ i.i.d.N(0,1) 
   bpT = 200; 
   e = randn(T,N); 
   b = [0.6,0; 0, 0.3]; 
   u = zeros(T+bpT, 2);
   u(:,1) = sqrt(1-b(1,1)^2)*randn(T+bpT, 1); 
   u(:,2) = sqrt(1-b(2,2)^2)*randn (T+bpT, 1); 
   F = zeros(T+bpT, 2); 
   for t = 2: T+bpT; 
       F(t,:) = F(t-1,:)* b + u(t, :) ;  
   end 
   X = F(bpT+1: bpT+T,:)* lam' + e; 
elseif DGP == 2 
   %% lam_i ~ i.i.d.N(0,1) fixed for each rep
   %% Ft = alp* F_{t-1} + u_it, u_it~ i.i.d.N(0,1)
   %% e_it = sig_i* v_it, v_it ~i.i.d.N(0,1), sig_i ~U(0.5, 1.5)
   bpT = 200;  
   b = [0.6,0; 0, 0.3]; 
   u = zeros(T+bpT, 2);
   u(:,1) = sqrt(1-b(1,1)^2)*randn(T+bpT, 1); 
   u(:,2) = sqrt(1-b(2,2)^2)*randn(T+bpT, 1);
   F = zeros(T+bpT, 2); 
   for t = 2: T+bpT; 
     F(t,:) = F(t-1,:)* b + u(t, :) ;  
   end 
   v = randn(T,N);
   sig = 1+ (1:1:N)'/(2*N);    % sigma_i ~ U(0.5,1.5)
   e = ones(T,1)*sig' .* v;
   X = F(bpT+1: bpT+T,:)* lam' + e; 
elseif DGP ==3 
   %% lam_i ~ i.i.d.N(0,1) fixed for each rep
   %% Ft = alp* F_{t-1} + u_it, u_it~ i.i.d.N(0,1)
   %% e_it = sig_i* v_it, v_it ~i.i.d.N(0,1), 
   %% sig_i^2 = (0.2+c_i) +(0.4+c_i)F_1t^2 + (0.2+c_i)*F_2t^2 
   bpT = 200; 
   b = [0.6,0; 0, 0.3];  
   u = zeros(T+bpT, 2);
   u(:,1) = sqrt(1-b(1,1)^2)*randn(T+bpT, 1); 
   u(:,2) = sqrt(1-b(2,2)^2)*randn(T+bpT, 1);
   F = zeros(T+bpT, 2); 
   for t = 2: T+bpT; 
     F(t,:) = F(t-1,:)* b + u(t, :) ;  
   end 
   v = randn(T,N);  
   sig2 = ones(T,1) * (0.2+sig)' + F(bpT+1: bpT+T,1).^2 *(0.1*ones(1,N))  + F(bpT+1: bpT+T,2).^2*(0.2*ones(1,N)) ;   
   e = sqrt(sig2).* v; 
   X = F(bpT+1: bpT+T,:)* lam' + e; 
elseif DGP ==4
    %% DGP.S4 
   %% cross sectional dependence + time invariant factor loadings 
   %% follow Chen et al. (2012, JoE) 
   ij = abs([1:1:N]'*ones(1,N) - ones(N,1)*[1:1:N]); 
   V = 0.5.^ij; 
   mu = zeros(N,1); 
   e = MultiNormal(mu, V, T); 
   bpT = 200;  
   b = [0.6,0; 0, 0.3]; 
   u = zeros(T+bpT, 2);
   u(:,1) = sqrt(1-b(1,1)^2)*randn(T+bpT, 1); 
   u(:,2) = sqrt(1-b(2,2)^2)*randn(T+bpT, 1); 
   F = zeros(T+bpT, 2); 
   for t = 2: T+bpT; 
       F(t,:) = F(t-1,:)* b + u(t, :) ;  
   end 
   X = F(bpT+1: bpT+T,:)* lam' + e; 
elseif DGP == 5 
    %% DGP.P1 
    %% abrupt structural change, similar as Han and Innoue (2014) 
   bpT = 200;   
   b = [0.6,0; 0, 0.3];  
   u = zeros(T+bpT, 2);
   u(:,1) = sqrt(1-b(1,1)^2)*randn(T+bpT, 1); 
   u(:,2) = sqrt(1-b(2,2)^2)*randn(T+bpT, 1);
   F = zeros(T+bpT, 2); 
   for t = 2: T+bpT; 
     F(t,:) = F(t-1,:)* b + u(t, :) ;  
   end  
   lam1 = lam(:,:,1); 
   lam2 = lam(:,:,2); 
   v = randn(T,N); 
   e = (ones(T,1)*sig') .* v;
   X = lam1 .* (F(bpT+1: bpT+T,1) *ones(1, N)) + lam2 .* (F(bpT+1: bpT+T,2) *ones(1, N)) + e;  
elseif DGP ==6 
    %% DGP.P2 
     %%  multiple structural changes-A nonpara. test for structural changes -DGP.P2
   bpT = 200;   
   b = [0.6,0; 0, 0.3]; 
   u = zeros(T+bpT, 2);
   u(:,1) = sqrt(1-b(1,1)^2)*randn(T+bpT, 1); 
   u(:,2) = sqrt(1-b(2,2)^2)*randn(T+bpT, 1);
   F = zeros(T+bpT, 2); 
   for t = 2: T+bpT; 
     F(t,:) = F(t-1,:)* b + u(t, :) ;  
   end  
   lam1 = lam(:,:,1); 
   lam2 = lam(:,:,2); 
   e = randn(T,N);  
   X = lam1 .* (F(bpT+1: bpT+T,1) *ones(1, N)) + lam2 .* (F(bpT+1: bpT+T,2) *ones(1, N)) + e;  
elseif DGP == 7
    %% DGP.P3 
    %% Smooth structural changes in factor loadings 
   bpT = 200;  
   b = [0.6,0; 0, 0.3]; 
   u = zeros(T+bpT, 2);
   u(:,1) = sqrt(1-b(1,1)^2)*randn(T+bpT, 1); 
   u(:,2) = sqrt(1-b(2,2)^2)*randn(T+bpT, 1);
   F = zeros(T+bpT, 2); 
   for t = 2: T+bpT; 
     F(t,:) = F(t-1,:)* b + u(t, :) ;  
   end 
    %e = 0.5* randn(T, N); 
    e =   randn(T, N); 
  
   X = F(bpT+1: bpT+T,1) *ones(1,N) .* lam(:,:,1) + F(bpT+1: bpT+T,2) *ones(1,N) .* lam(:,:,2)  + e; 
elseif DGP == 8
    %% DGP.P4 
     %% Multiple structural changes: A nonparametric paper 
   bpT = 200;   
   b = [0.6,0; 0, 0.3];  
   u = zeros(T+bpT, 2);
   u(:,1) = sqrt(1-b(1,1)^2)*randn(T+bpT, 1); 
   u(:,2) = sqrt(1-b(2,2)^2)*randn(T+bpT, 1);
   F = zeros(T+bpT, 2); 
   for t = 2: T+bpT; 
     F(t,:) = F(t-1,:)* b + u(t, :) ;  
   end  
   lam1 = lam(:,:,1); 
   lam2 = lam(:,:,2); 
   e = randn(T,N);  
   X = lam1 .* (F(bpT+1: bpT+T,1) *ones(1, N)) + lam2 .* (F(bpT+1: bpT+T,2) *ones(1, N)) + e; 
elseif DGP == 9 
    %% DGP.P5 
     %% Single structural break + cross sectional dependence 
     ij = abs([1:1:N]'*ones(1,N) - ones(N,1)*[1:1:N]); 
   V = 0.5.^ij; 
   mu = zeros(N,1); 
   e = MultiNormal(mu, V, T); 
    bpT = 200;  
   b = [0.6,0; 0, 0.3]; 
   u = zeros(T+bpT, 2);
   u(:,1) = sqrt(1-b(1,1)^2)*randn(T+bpT, 1); 
   u(:,2) = sqrt(1-b(2,2)^2)*randn(T+bpT, 1);
   F = zeros(T+bpT, 2); 
   for t = 2: T+bpT; 
     F(t,:) = F(t-1,:)* b + u(t, :) ;  
   end  
   X = F(bpT+1: bpT+T,1) *ones(1,N) .* lam(:,:,1) + F(bpT+1: bpT+T,2) *ones(1,N) .* lam(:,:,2)  + e; 
elseif DGP == 10 
    %% DGP.P6 
    %%  multiple structural changes+cross sectional dependence -A nonpara. test for structural changes -DGP.P2
     ij = abs([1:1:N]'*ones(1,N) - ones(N,1)*[1:1:N]); 
   V = 0.5.^ij; 
   mu = zeros(N,1); 
   e = MultiNormal(mu, V, T); 
    bpT = 200;  
   b = [0.6,0; 0, 0.3]; 
   u = zeros(T+bpT, 2);
   u(:,1) = sqrt(1-b(1,1)^2)*randn(T+bpT, 1); 
   u(:,2) = sqrt(1-b(2,2)^2)*randn(T+bpT, 1);
   F = zeros(T+bpT, 2); 
   for t = 2: T+bpT; 
     F(t,:) = F(t-1,:)* b + u(t, :) ;  
   end  
   X = F(bpT+1: bpT+T,1) *ones(1,N) .* lam(:,:,1) + F(bpT+1: bpT+T,2) *ones(1,N) .* lam(:,:,2)  + e; 

elseif DGP == 11
    %% DGP.P7 
     %%  cross sectional dependence + time varying factor loadings 
   ij = abs([1:1:N]'*ones(1,N) - ones(N,1)*[1:1:N]); 
   V = 0.5.^ij; 
   mu = zeros(N,1); 
   e = MultiNormal(mu, V, T); 
    bpT = 200;  
   b = [0.6,0; 0, 0.3]; 
   u = zeros(T+bpT, 2);
   u(:,1) = sqrt(1-b(1,1)^2)*randn(T+bpT, 1); 
   u(:,2) = sqrt(1-b(2,2)^2)*randn(T+bpT, 1);
   F = zeros(T+bpT, 2); 
   for t = 2: T+bpT; 
     F(t,:) = F(t-1,:)* b + u(t, :) ;  
   end 
   % e = randn(T, N); 
   X = F(bpT+1: bpT+T,1) *ones(1,N) .* lam(:,:,1) + F(bpT+1: bpT+T,2) *ones(1,N) .* lam(:,:,2)  + e; 
elseif DGP == 12 
    %% DGP.P8 
    %% heterogenous time variations , DGP.P8 in ET's paper 
     bpT = 200;   
   b = [0.6,0; 0, 0.3]; 
   u = zeros(T+bpT, 2);
   u(:,1) = sqrt(1-b(1,1)^2)*randn(T+bpT, 1); 
   u(:,2) = sqrt(1-b(2,2)^2)*randn(T+bpT, 1);
   F = zeros(T+bpT, 2); 
   for t = 2: T+bpT; 
     F(t,:) = F(t-1,:)* b + u(t, :) ;  
   end  
   lam1 = lam(:,:,1); 
   lam2 = lam(:,:,2); 
   e = randn(T,N);  
   X = lam1 .* (F(bpT+1: bpT+T,1) *ones(1, N)) + lam2 .* (F(bpT+1: bpT+T,2) *ones(1, N)) + e;  
elseif DGP == 13 
    %% DGP.P9, local power 1
     bpT = 200;   
   b = [0.6,0; 0, 0.3];  
   u = zeros(T+bpT, 2);
   u(:,1) = sqrt(1-b(1,1)^2)*randn(T+bpT, 1); 
   u(:,2) = sqrt(1-b(2,2)^2)*randn(T+bpT, 1);
   F = zeros(T+bpT, 2); 
   for t = 2: T+bpT; 
     F(t,:) = F(t-1,:)* b + u(t, :) ;  
   end  
   lam1 = lam(:,:,1); 
   lam2 = lam(:,:,2); 
   v = randn(T,N); 
   e = (ones(T,1)*sig') .* v;
   X = lam1 .* (F(bpT+1: bpT+T,1) *ones(1, N)) + lam2 .* (F(bpT+1: bpT+T,2) *ones(1, N)) + e;   
elseif DGP ==14
    %% DGP.P10, local power 2 
   bpT = 200;  
   b = [0.6,0; 0, 0.3]; 
   u = zeros(T+bpT, 2);
   u(:,1) = sqrt(1-b(1,1)^2)*randn(T+bpT, 1); 
   u(:,2) = sqrt(1-b(2,2)^2)*randn(T+bpT, 1);
   F = zeros(T+bpT, 2); 
   for t = 2: T+bpT; 
     F(t,:) = F(t-1,:)* b + u(t, :) ;  
   end 
   e = randn(T, N); 
   X = F(bpT+1: bpT+T,1) *ones(1,N) .* lam(:,:,1) + F(bpT+1: bpT+T,2) *ones(1,N) .* lam(:,:,2)  + e; 
elseif DGP == 15 
    %% DGP. P11 
    %% structural change at the endpoint -ET- R&R, c= 0.5 
    bpT = 200;   
   b = [0.6,0; 0, 0.3];  
   u = zeros(T+bpT, 2);
   u(:,1) = sqrt(1-b(1,1)^2)*randn(T+bpT, 1); 
   u(:,2) = sqrt(1-b(2,2)^2)*randn(T+bpT, 1);
   F = zeros(T+bpT, 2); 
   for t = 2: T+bpT; 
     F(t,:) = F(t-1,:)* b + u(t, :) ;  
   end  
   lam1 = lam(:,:,1); 
   lam2 = lam(:,:,2); 
  v = randn(T,N); 
   e = (ones(T,1)*sig') .* v;
   X = lam1 .* (F(bpT+1: bpT+T,1) *ones(1, N)) + lam2 .* (F(bpT+1: bpT+T,2) *ones(1, N)) + e; 
end  
Ftrue = F(bpT+1: bpT+T,:); 
end
   

    





