function [supLM, supW, crit] = HI_test(Yobs, r, tau)
%% This function calculate Han and Inoue's (2014, ET) test for
%%  parameter instability in dynamic factor models 
%% Witten by Xia Wang,  Dec. 2014, @SMU, Singapore
%% Input: yobs: data, dimension: T x N 
%%             r: number of factor
%%            tau: trimming parameter 

%% Output: supLM: supLM test statistic , dim: 1 x 1 
%%                supW: supW test statistic, dim: 1 x 1
%%                crit: The corresponding Andrews's (1993) critical value [5%, 10%], dim: 1 x 2  

[T, N] = size(Yobs); 
%% Step 1: Estimate common factor and factor loadings by PCA 
%%       Yobs = F * lam' + e, Yobs: T x N;   F: T x r;    lam: N x r
[T, N] = size(Yobs); 
YY = Yobs * Yobs' /(T*N);                      % dim: T x T 
[Ffit0, D] = eig(YY); 
F = Ffit0(:, T-r+1:T) * sqrt(T);              % dim: T x r, estimated factors by PCA 
lam = Yobs' * F/T;                                % dim: T x r, estimated factor loadings 


%% Step 2: calculate A(tau, F) , Shat and Stid 
T0 = floor(tau*T)+1;      % started date 
T1 = floor((1-tau)*T)+1;  % ended date 
A = zeros(T1-T0+1, r*(r+1)/2); 

% Construct vech(Ft * Ft' - I) 
% recall: Ft: T x r; vech(Ft* Ft' -I): T x r(r+1)/2 
J1 = F;                                    % dim: T x r 
J2 = F(:, 1) * ones(1, r);        % dim: T x r 
Er = eye(r) ;                               % dim: r x r 
I= Er(:,1) ;

for i = 1: r-1 
    J1 = [J1, F(:, i+1: r) ];                   % final dim: T x r(r+1)/2
    J2 = [J2, F(:, i+1)* ones(1,  r-i) ];   % final dim: T x r(r+1)/2 
    Er = eye(r) ;                               % dim: r x r 
    I  =   [ I; Er(i+1:r, i+1)];                        % final dim: r(r+1)/2 x 1 
end 
FF= J1.* J2 - ones(T,1) * I';         % dim: T x r(r+1)/2

 %  S = ceil(T^(1/5));                              % remark: may not be the para. selected by Han and Inoue (2014) 
  S = ceil(T^(1/3)); 
 w = 1- [1: 1: S]' /(S);               % dim: S x 1, Bartlett kernel 
 Omg = T^(-1) * FF(1:T,:)' * FF(1:T,:) ;        %dim: r(r+1)/2 x r(r+1)/2
for j = 1: S
     Gj = (T)^(-1) * FF(j+1:T,:)' * FF(1: T-j,:); 
     Omg = Omg + w(j)* (Gj + Gj'); 
end 

W = zeros(T1-T0+1,1); 
LM = zeros(T1-T0+1,1); 

for t = T0: T1
    A1 = F(1: t, :)' * F(1: t, :)/t;                       % dim: r x r 
    A2 = F(t+1: T,:)' * F(t+1: T,:)/(T-t);          % dim: r x r 
    Ad = tril(sqrt(T) * (A1 - A2)) ;                  % dim: r x r 
    At = nonzeros(Ad);                              % dim: r(r+1)/2 
    A(t-T0+1,:)  = At'; 
    
       S1 = ceil( t^(1/5) );                       % remark: may not be the para. selected by Han and Inoue (2014) 
      S2 = ceil( (T-t)^(1/5) );                   % remark: may not be the para. selected by Han and Inoue (2014) 
   
   
  % S1 = ceil( t^(1/3) );                       % remark: may not be the para. selected by Han and Inoue (2014) 
  %   S2 = ceil( (T-t)^(1/3) );                   % remark: may not be the para. selected by Han and Inoue (2014) 

   
   
    w1 = 1- [1: 1: S1]' /(S1);        % dim: S1 x 1, Bartlett kernel 
    w2 = 1- [1: 1: S2]' /(S2);        % dim: S2 x 1, Bartlett kernel 
  
    Omg1 = t^(-1) * FF(1:t,:)'* FF(1:t,:);           % dim: r(r+1)/2 x r(r+1)/2 
    Omg2 = (T-t)^(-1) * FF(t+1:T,:)' * FF(t+1:T,:) ;        %dim: r(r+1)/2 x r(r+1)/2
    
    for j = 1: min(t, S1)
        G1j = t^(-1) * FF(j+1: t,:)' * FF(1: t-j,:); 
        Omg1 = Omg1 + w1(j)* (G1j + G1j'); 
    end 
    for j = 1:min(T-t-1, S2)
        G2j = (T-t)^(-1) * FF(j+t+1:T,:)' * FF(t+1: T-j,:); 
        Omg2 = Omg2 + w2(j)* (G2j + G2j'); 
    end 
     
    p = t/T; 
    
    Stld = (1/p + 1/(1-p)) * Omg;  
    Shat = 1/p* Omg1 + 1/(1-p) * Omg2;  
     
    Wt = At' * pinv(Shat) * At; 
    LMt = At' * pinv(Stld) * At; 
    W(t-T0+1) = Wt; 
    LM(t-T0+1) = LMt; 
end 


supLM = max(LM);                %dim: 1 x 1
supW = max(W);              %dim: 1 x 1 

%% Step 5: Get Andrews' (1993) critical value 
% we only import tau = 0.25. 0.20, 0.15, 0.10, 0.05 
%                           r = 1, 2 3, .....8 

CritValue5 = [ 7.93, 10.75, 13.16, 15.34, 17.25, 19.07, 20.63, 20.95, 24.31, 25.80, 27.43, 28.98;
                         8.45, 11.26, 13.69, 15.84, 17.88, 19.64, 21.07, 21.47, 24.91, 26.42, 27.93, 29.61;
                         8.85, 11.79, 14.15, 16.45, 18.35, 20.26, 21.84, 22.13, 25.47, 27.03, 28.55, 30.16;
                         9.31, 12.27, 14.62, 16.98, 18.93, 20.82, 22.51, 22.87, 26.16, 27.87, 29.21, 30.88;
                         9.84, 12.93, 15.15, 17.56, 19.61, 21.56, 23.22, 23.60, 26.94, 28.63, 30.15, 31.76 ];
                     
 CritValue10 = [6.46, 9.10, 11.29,13.36, 15.17, 17.00, 18.38, 18.61, 21.93, 23.32, 24.82, 26.37;
                           6.80, 9.59, 11.80, 13.82, 15.63, 17.56, 19.04, 19.17, 22.54, 24.00, 25.41, 27.08;
                           7.17,10.01, 12.27, 14.31, 16.20, 18.12, 19.69, 19.82, 23.15, 24.62, 26.14, 27.74;
                           7.63, 10.50, 12.81, 14.94, 16.87, 18.78, 20.32, 20.45, 23.77, 25.39, 26.86, 28.47;
                           8.19, 11.20, 13.47, 15.62, 17.69, 19.49, 21.02, 21.23, 24.64, 26.24, 27.91, 29.31 ];
                       
 kk = round((0.3 - tau)/0.05) ;
 
 crit = [CritValue5(kk, r*(r+1)/2), CritValue10(kk,r*(r+1)/2)]; 

 