function [ b ] = estimate_local_stationary( Yobs, Xobs, h, kern, p, method)
%% This function is used to estimate the local stationary process 
%% y_t = x_t' * a(t/T) + eps_t
%% by using the p-th order local polynomial process 
%% Written by Xia WANG, 2016-2-10 @ Guangzhou

%% Input: Yobs: observations of y_t, dim: T x 1 
%%             Xobs: observations of x_t, dim: T x N 
%%             h: bandwidth 
%%             kern: kernal function 
%%             kern =1; Uniform kernel 
%%             kern =2; Epanechnikov kernel 
%%             kern =3; Quatic kernel 
%%             p: order of polynomial 
%%            Method: methods deal with the boundary problem 
%%                          = 1: boundary kernel 
%%                          = 2: reflection method 

%% Output:  b -- estimated time varying coefficient, dim: T x N

[T, N] = size(Xobs); 
 
bound = ceil(T*h); 
 b = zeros(T, N); 
 
if kern == 1           %uniform kernel 
    fk = @(x) 0.5; 
elseif  kern ==2                    % Epan. kernel 
    fk = @(x) 0.75 * (1- x.^2) ; 
elseif kern ==3                     % Quatic kernel 
    fk = @(x) 15/16 * (1-x.^2)^2 ;
end 

%%  Method 1: use the boundary kernel 
if method == 1
    tidx = (1:1:T)';  
   w =( tidx *ones(1,T) - ones(T,1) * tidx' )./(T*h);  %% (t-r)/Th
   K = kernel(kern, w);                   % dim: TxT (t,r) 
    for rr = 1: bound
        if kern ==1
            Qv = 0.5*(1+rr/(T*h));
        else
            Qv = integral(fk, -rr/(T*h), 1);
        end
        K(:, rr) = K(:,rr)/Qv;
    end
    
    for rr = T-bound:T
        if kern ==1
            Qv = 0.5*((1-rr/T)/h+1);
        else
            Qv = integral(fk, -1, (1-rr/T)/h);
        end
           K(:, rr) = K(:,rr)/Qv;
    end
   
    for t = 1 : T
        tx = (tidx-t) / T;               % dim: T x 1
        ZZ = Xobs;
        for ii = 1: p
            temp = (tx.^ii* ones(1,N)) .* Xobs;     % dim: T x N
            ZZ = [ZZ, temp];             % final dim of ZZ: T x N(p+1)
        end
        W  = diag(K(:, t)/h); 
        alp = inv(ZZ' * W * ZZ) * ZZ' * W * Yobs;       % dim: N(p+1) x 1 
        b(t, :) = alp(1: N, 1)';  
    end
    
    %% Method 2: reflection method
elseif method == 2
    % left boundary region
    xleft = flipud(Xobs(2:bound, :));
    yleft =  flipud(Yobs(2:bound, :));
    % right boundary region
    xright= flipud(Xobs(T-bound+1:T-1, :));
    yright= flipud(Yobs(T-bound+1:T-1, :));
     
    % modified data
    Xobssym = [xleft; Xobs; xright];     %dim: (T + 2*bound - 2) x N
    Yobssym = [yleft; Yobs; yright];      %dim: (T + 2*bound - 2) x 1
    % generate t/T
    tidx = (1:1: T)' ; 
    tidxleft= (-bound: 1: -2)'; 
    tidxright = (T+1: 1: T + bound-1)'; 
    tbidx = [tidxleft; tidx;  tidxright];
    [Tb, temp]= size(tbidx);
    wb =( tidx *ones(1,Tb) - ones(T,1) * tbidx' )./(T*h);     %dimension  T*Tb
    K = kernel(kern, wb);                   % dim: TxTb (t,r) 
    for t = 1 : T
        tx = (tbidx-t) / T;               % dim: Tb x 1
        ZZ = Xobssym;         % dim: Tb x N 
        for ii = 1: p
            temp = (tx.^ii* ones(1,N)) .* Xobssym;     % dim: Tb x N
            ZZ = [ZZ, temp];             % final dim of ZZ: Tb x N(p+1)
        end
        W  = diag(K(t, :)/h);               % dim: Tb x Tb
        alp = inv(ZZ' * W * ZZ) * ZZ' * W *  Yobssym;       % dim: N(p+1) x 1 
        b(t, :) = alp(1: N, 1)';  
    end
    
    
end
 

end

