function [lam0, lam,  F0, SM ] = mod_test_tvloading(Xobs, R, h , kern)
%% This function is used to calculate Su and Wang's test for time varying
%% factor loadings 
%% Paper: Testing for structural changes in factor models via a nonparametric regression 
%% Input: Xobs: data matrix, TxN 
%%        R: number of factors 
%%        h: bandwidth 
%%        kern: kernel function, kern =1; Uniform kernel 
%%              kern =2; Epanechnikov kernel 
%%              kern =3; Quatic kernel

%% Output: standardized test statistic 
%% Written by Xia WANG, SMU, Singapore, Dec. 2014
 
if kern ==1; 
    K1= 1/2;   % K1 =int[K(u)^2]du 
    K2 = 7/24;  % K2 = int[\int K(t)K(t+u)dt]^2 du 
elseif kern ==2; 
   %  K2 = 2* 413113/623700*(9/16)^2 ; 
   K1 =3/5; 
   K2 =0.4191; 
elseif kern ==3; 
   % K2 = 151761218615/593300291584*2; 
   K1 =  (1-4/3+6/5-4/7+1/9)*2*(15/16)^2;
   % K1 =0.714286;
    K2 = 0.5116; 
end

[T, N] = size(Xobs); 

%% Step 1: estimate the time invariant factor loadings 
% F0: estimated common factors using classical PCA 
% lam0: time-invariant factor loadings 

XX = Xobs *Xobs'/(T*N);         % dim: T x T 
[Ffit0, D] = eig(XX); 
F0 = Ffit0(:, T-R+1:T)*sqrt(T); % dim: TxR, estimated factors by PCA
lam0=  Xobs' * F0/T;            % dim: NxR, estimated factor loadings 
eps0 = Xobs- F0 * lam0';        % dim: TxN, error term from time invariant factor models

%% Step 2: estimate time varying factor loadings
% Remark: It is not the true estimate of time varying factor loadings, but 
% the local estimate of lam_i by local OLS uing the regression: 
% k_st^(1/2)*X_is = k_st^(1/2)* F_s'* lam_it + k_st^(1/2) * e_is 

% step 2.1 construct Hong and Li's (2005) boundary kernel 
tidx = 1:1:T; 
tidx = tidx'/T; 
w =( tidx *ones(1,T) - ones(T,1) * tidx' )./h;  %% (t-r)/Th
K = kernel(kern, w);                   % dim: TxT (t,r) 
bound = ceil(T*h); 
 
Kbar =  (3/5 -3/4* w.^2 +3/8* abs(w).^3 -3/160 * abs(w).^5).* (abs(w) <2);  % Convolution kernel for Epanechnikov kernel, dim: T x T  
 
if kern ==1 
    fk = @(x) 0.5 ;
elseif kern ==2
    fk = @(x) 0.75 * (1- x.^2) ; 
elseif kern ==3 
    fk = @(x) 15/16 * (1-x.^2)^2 ;
end 

for rr = 1: bound 
    if kern ==1
        Qv = 0.5*(1+rr/(T*h)); 
    else
        Qv = integral(fk, -rr/(T*h), 1);
    end
    K(:, rr) = K(:,rr)/Qv; 
end 

for rr = T-bound:T
    if kern ==1 
        Qv = 0.5*((1-rr/T)/h+1); 
    else
        Qv = integral(fk, -1, (1-rr/T)/h);
    end
    K(:, rr) = K(:,rr)/Qv;      
end 

% step 2.2: construct the estimation lam_it = (\sum_{s=1}^T F_s F_s'
% K_st)^{-1} (\sum_{s=1}^T F_s X_is K_st)

lam = zeros(T, N, R);

B1 = 0; 
B2 = 0; 

for t = 1:T
    ST = F0'*(F0.*(K(:,t)*ones(1,R)))/(T*h);        % dim: R x R 
    % estimate lam_it 
    lamt = pinv(ST) * ((F0 .* (K(:,t)*ones(1,R)))' * Xobs/(T*h));  % dim: R x N 
    lam(t,:,:) = lamt';    
    % calculate the bias terms B1 and B2  
    B1 = B1 + T^(-2) * N^(-1/2) * h^(-3/2) * sum(eps0.^2, 2)' *(sum(F0 * pinv(ST) * pinv(ST) .* F0, 2) .* K(:,t).^2); 
    B2 = B2 -2 * T^(-2) * N^(-1/2) * h^(-1/2) *  sum(eps0.^2, 2)' *(sum(F0 * pinv(ST) .* F0, 2) .* K(:,t)); 
end 

B3 = T^(-1) * N^(-1/2) * h^(1/2) * sum(F0.^2, 2)' * sum(eps0.^2, 2); 

 

B = B1 + B2 + B3; 
  
%% step 3: construct test statistic SM 
% lam0: N x R; lam: T x N x R 

% step 3.1: calculate sum(sum(lam_it- lam_i0)'(lam_it-lam_i0))) 
temp = ones(1,T); 
temp2 = kron(lam0, temp)';      % dim:  (T*R)x N 
lam0T = permute(reshape(temp2, T, R, N), [1,3,2]);   %dim: T x N x R 
 
M = sum(sum(sum((lam-lam0T).^2 , 3), 2))/(N*T); 

% step 3.2: calculate the variance term 
S = F0'* F0 /T;           % dim: R x R 
S2 = pinv(S) * pinv(S)'; 


VT =   Kbar.^2 .* (F0 *S2 * F0') .* (F0 *S2* F0') .* (eps0 *eps0') .^2 ; 
 
 V  = 2 * T^(-2) * N^(-1) * h^(-1) *(sum(sum(VT)) - sum(diag(VT)));

  

% step 3.3 calculate the standardized test statistic
SM = (T *N^(1/2) *h^(1/2)*M - B) /sqrt(V);
 
end

