function [SM1, SM2,SM3, SM4, lam0, F0] = test_common_component(Xobs, R, h , kern)
 %% This function is used to calculate Su and White's (2014) test for time varying factor model 
 %% On Time Varying Factor Model: Estimation and Inference 
 %% This paper focus on testing the common component 
 %% Input: Xobs: data matrix, TxN 
 %%        R: number of factors 
 %%        h: bandwidth 
 %%        kern: kernel function, kern =1; Uniform kernel 
 %%              kern =2; Epanechnikov kernel 
 %%              kern =3; Quatic kernel
 
 %% Output: Standardized test statistic
 %% Written by Xia WANG, @SMU, Singapore, January, 2015 
 
 [T, N] = size(Xobs); 
 
if kern ==1 
    K1= 1/2;   % K1 =int[K(u)^2]du 
    K2 = 7/24;  % K2 = int[\int K(t)K(t+u)dt]^2 du 
elseif kern ==2 
   K1 =3/5; 
   K2 =0.4191; 
elseif kern ==3  
   K1 =  (1-4/3+6/5-4/7+1/9)*2*(15/16)^2;
   % K1 =0.714286;
   K2 = 0.5116; 
end

%% Step 1: estimate the time invariant factor model, and get the estimated
%% common component
XX = Xobs *Xobs'/(T*N);         % dim: T x T 
[Ffit0, D] = eig(XX); 
F0 = Ffit0(:, T-R+1:T)*sqrt(T); % dim: TxR, estimated factors by PCA
lam0 =  Xobs' * F0/T;            % dim: NxR, estimated factor loadings 

LF0 = F0* lam0'; 

e = Xobs - LF0; 
%% Step 2: estimate the time varying factor model, and get the estimated 
%% common component 

%% step 2.1 construct Hong and Li's (2005) boundary kernel 
tidx = 1:1:T; 
tidx = tidx'/T; 
w =( tidx *ones(1,T) - ones(T,1) * tidx' )./h;  %% (t-r)/Th
K = kernel(kern, w);                   % dim: TxT (t,r) 
bound = ceil(T*h); 
 

if kern ==1 
    fk = @(x) 0.5 ;
elseif kern ==2
    fk = @(x) 0.75 * (1- x.^2) ; 
elseif kern ==3 
    fk = @(x) 15/16 * (1-x.^2)^2 ;
end 

%% WX: 2015-4-20, if we do not use the boundary kernel
% 
% for rr = 1: bound 
%     if kern ==1
%         Qv = 0.5*(1+rr/(T*h)); 
%     else
%    %      Qv = integral(fk, -rr/(T*h), 1);
%    Qv = quad(fk, -rr/(T*h), 1);
%     end
%     K(:, rr) = K(:,rr)/Qv; 
% end 
% 
% for rr = T-bound:T
%     if kern ==1 
%         Qv = 0.5*((1-rr/T)/h+1); 
%     else
%     %    Qv = integral(fk, -1, (1-rr/T)/h);
%     Qv = quad(fk, -1, (1-rr/T)/h);
%     end
%     K(:, rr) = K(:,rr)/Qv;      
% end 
 
%% Step 2.2: local principal component analysis 
lam = zeros(T, N, R); 
Fest = zeros(T, T, R); 
Fb = zeros(T, R); 
LF_tv = zeros(T, N); 
   etld = zeros(T, N); 
for t = 1 : T
    %% first stage, estimate the time varying factor model by nonparametric smoothing 
    Kt = h^(-1/2)* K(:,t).^(1/2);              % dim: T x 1
    Xtld = repmat(Kt, 1, N) .* Xobs;           % dim: T x N
    X2 = Xtld* Xtld'/(N*T);                     % dim: T x T
    [EV, D] = eig(X2);                         % EV's dim: T x T, D: diag, 
    Ftild = EV(:,T-R+1: T)*sqrt(T);            % dim: T x R 
    lamtild = (Ftild' * Xtld/T)';              % dim: N x R 
    lam(t,:,:) = lamtild;
    Fest(t,:,:) = Ftild;
    %% Second stage, estimate the common factors by regress Xit on lam_it
    Fb(t,:) = Xobs(t,:) * lamtild * pinv(lamtild'* lamtild)' ; % dim: 1 x R 
    %% Calculate the common component 
    LF_tv(t, :) =  Fb(t,:)* lamtild';        % dim: 1 x N 
    etld(t,:) = Xobs(t,:) - LF_tv(t,:); 
end

%% Step 3: Construct test statistic 

 M = sum(sum((LF_tv - LF0).^2,1))/(N * T); 
 
 % Bias term 
 B = h^(1/2)* T^(-2) * N^(-1/2)*sum( ((1/h*K .* (F0* F0')- F0 * F0').^2) * sum(e.^2, 2)); 
 
 B2 = h^(1/2)* T^(-2) * N^(-1/2)*sum( ((1/h*K .* (Fb* Fb')- Fb * Fb').^2) * sum(etld.^2, 2)); 

 B3 = h^(1/2)* T^(-2) * N^(-1/2)*sum( ((1/h*K .* (Fb* Fb')- F0 * F0').^2) * sum(etld.^2, 2)); 

 % Variance term 
 Kbar =  (3/5 -3/4* w.^2 +3/8* abs(w).^3 -3/160 * abs(w).^5).* (abs(w) <2);  % Convolution kernel for Epanechnikov kernel, dim: T x T  
 sig_F = F0'* F0/T ;               % dim: R x R 
 
 sig_Fb = Fb'*Fb/T;
 
 VTT1 =   Kbar.^2 .* (F0 * sig_F * F0').^2 .* ((e.^2)* (e.^2)'); 
 
 V1 = 2 * T^(-2) * N^(-1) * h^(-1) *(sum(sum(VTT1)) - sum(diag(VTT1)));
 
 VTT2 =   Kbar.^2 .* (F0 * sig_F * F0').^2 .* (e * e').^2; 
 
 V2 = 2 * T^(-2) * N^(-1) * h^(-1) *(sum(sum(VTT2)) - sum(diag(VTT2)));
 
 VTT3 =   Kbar.^2 .* (Fb * sig_Fb * Fb').^2 .* ((etld.^2)* (etld.^2)'); 
 
 V3 = 2 * T^(-2) * N^(-1) * h^(-1) *(sum(sum(VTT3)) - sum(diag(VTT3)));
 
 VTT4 =   Kbar.^2 .* (Fb * sig_Fb * Fb').^2 .* (etld * etld').^2; 
 
 V4 = 2 * T^(-2) * N^(-1) * h^(-1) *(sum(sum(VTT4)) - sum(diag(VTT4)));

 
 SM1 = (T * N^(1/2)* h^(1/2)*M - B)* V1^(-1/2); 
 
 SM2 = (T * N^(1/2)* h^(1/2)*M - B)* V2^(-1/2);
 
 SM3 = (T * N^(1/2)* h^(1/2)*M - B2)* V3^(-1/2); 
 
 SM4 = (T * N^(1/2)* h^(1/2)*M - B2)* V4^(-1/2);
 
 
%  [T * N^(1/2)* h^(1/2)*M, B, B2, B3, V1, V2, V3, V4 SM1, SM2, SM3, SM4] 

end